#!/usr/bin/env python

#
# COMMENT / DESCRIPTION #
#

import os

from PyQt5 import QtCore, QtGui, QtWidgets

import enums
import gui.panels
import gui.scene
import gui.paths ###
import gui.view

class Alert(QtWidgets.QMessageBox):
    def __init__(self, title, message):
        super().__init__()
        self.setIcon(QtWidgets.QMessageBox.Warning)
        self.setWindowFlags(QtCore.Qt.Popup)
        self.setText("<b>"+title+"</b><br>"+message)

    def exec(self):
        QtWidgets.QApplication.setOverrideCursor(QtCore.Qt.ArrowCursor)
        rtn = super().exec()
        QtWidgets.QApplication.restoreOverrideCursor()
        return rtn


class Window(QtWidgets.QMainWindow):
    def __init__(self):
        super().__init__()

        self.setWindowTitle("Visdan")
        self.setMinimumSize(800, 600)

        location = os.path.dirname(os.path.realpath(__file__))
        QtGui.QFontDatabase.addApplicationFont(location+'/resources/fonts/Ubuntu-R.ttf')
        self.setFont(QtGui.QFont('Ubuntu'))

        self.setCorner(QtCore.Qt.BottomRightCorner,
                       QtCore.Qt.RightDockWidgetArea)

        ### VIEW
        self.view = gui.view.View()
        self.setCentralWidget(self.view)

        ### SCENE
        self.scene = gui.scene.Scene()
        self.view.setScene(self.scene)
       
        ### DEFINED PATHS 
        self.paths = gui.paths.DefinedPaths()
        self.addDockWidget(QtCore.Qt.BottomDockWidgetArea, self.paths)

        ### SIDEBAR S PANELY
        self.sidebar = gui.panels.Sidebar()
        self.addDockWidget(QtCore.Qt.RightDockWidgetArea, self.sidebar)

    def resizeEvent(self, event):
        ## nechat tady, nebo vyslat umely signal a do controlleru?
        ## mozna tady, prece jen to neni az tolik uzivatelsky vstup?
        self.sidebar.resize_button()
        self.sidebar.setMaximumWidth(event.size().width()/2)

    def alert(self, alert_message):
        messages = {}
        messages[enums.Alert.first_node_first] = ["First node first",
            "First end node must be selected before any datapaths "
            "can be added to the path."]
        messages[enums.Alert.datapath_before_end_node] = [
            "Datapath before end node",
            "At least one datapath must be selected before the "
            "end node can be added."]
        messages[enums.Alert.selected_must_neighbor] = [
            "Selected must neighbor",
            "The selected element must neighbor with the preceeding one."]
        messages[enums.Alert.node_inactive] = ["Node inactive",
            "Some of the selected nodes are not active anymore. "
            "The path will be discarded."]
        messages[enums.Alert.datapath_inactive] = ["Datapath inactive",
            "Some of the selected datapaths are not active anymore. "
            "The path will be discarded."]
        messages[enums.Alert.connection_timed_out] = [
            "Connection timed out",
            "Connection to the controller timed out. Please try again."]
        messages[enums.Alert.connection_failed] = [
            "Connection failed",
            "Connection to the controller failed. "
            "Please check all fields are filled correctly."]
        messages[enums.Alert.connection_closed] = [
            "Connection closed",
            "Connection to the controller was closed. "
            "Please check your connection and try to connect again."]
        messages[enums.Alert.connection_lost] = [
            "Connection lost",
            "Connection to the controller was lost. "
            "Please try to connect again."]
        messages[enums.Alert.path_definition_failed] = [
            "Path definition failed",
            "An error occured when implementing the defined path."]
        messages[enums.Alert.internal_error] = [
            "Internal error",
            "An unexpected internal error occured."]

        alert = Alert(messages[alert_message][0], messages[alert_message][1])
        alert.exec()
        
    def alert_question(self, alert_message):
        messages = {}
        messages[enums.AlertQuestion.start_all_over] = ["Start all over",
            "Removing the first node will clear the whole path. "
            "Do you want to continue?"]
        messages[enums.AlertQuestion.delete_all_succeeding] = [
            "Delete all succeeding",
            "Removing the datapath will also remove all succeding elements "
            "in the path. Do you want to continue?"]
        messages[enums.AlertQuestion.discard_defined_path] = [
            "Discard path",
            "Are you sure you want to discard the whole path?"]

        alert = Alert(messages[alert_message][0], messages[alert_message][1])
        alert.setStandardButtons(QtWidgets.QMessageBox.Ok |
            QtWidgets.QMessageBox.Cancel)
        alert.setDefaultButton(QtWidgets.QMessageBox.Ok)

        return True if alert.exec() == QtWidgets.QMessageBox.Ok else False

    def save_file(self):
        return QtWidgets.QFileDialog.getSaveFileName(self,
            "Save data", QtCore.QDir.homePath(), "JSON (*.json)")
    
    def open_file(self):
        return QtWidgets.QFileDialog.getOpenFileName(self,
            "Load data", QtCore.QDir.homePath(), "JSON (*.json)")
